package uk.ac.strath.esru;

import javax.swing.JDialog;
import javax.swing.JTextField;

// This file contains code for the following:
// 1 Check relevance of numerical input i.e. range checking
// 2 Make additional checks as to interdependencies of inputs
// 3 Add event to UndoStack
// 4 Add event to console output
// 5 Set unsaved information flag

public class AcceptInput{
	
	JTextField jtf;
	Language l = new Language();
	public String message = null;
	String button = l.OK;
	String msgHead = "";
	int dummy = 0;
	
	// Check floating point number ranges
	public boolean CheckInput(int integerChecks, JTextField field, int index){
		this.jtf = field;
		int input = 0;
		if(jtf.getText().equalsIgnoreCase(""))return true; // return if field is blank

		if(!(index == -1)){ // No checks made if index = -1
			try{
				input = Integer.valueOf(jtf.getText());
			} catch (Exception e){
				jtf.setText("");
			}
			
			if(!RangeCheck(input, index)){
				try{
					UserDialog dialog = new UserDialog(button, dummy, msgHead, message);
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setTitle(l.warning);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
				return false;
			}
		}
		
		AddEventToUndoStack();
		AddEventToConsole();
		InOut.inputsModified = true;
		
//		System.out.println("input accepted as " + input);
		return true;
	}
	

	public boolean CheckInput(double doubleChecks, JTextField field, int index){
		this.jtf = field;
		double input = 0;
		String button = l.OK;
		String msgHead = "";
		int dummy = 0;
		if(jtf.getText().equalsIgnoreCase(""))return true; // return if field is blank
		try{
			//input = Integer.valueOf(jtf.getText());//
			input = Double.valueOf(jtf.getText()).doubleValue();
		} catch (Exception e){
			jtf.setText("");
		}
		
		if(!RangeCheck(input, index)){
			try{
				UserDialog dialog = new UserDialog(button, dummy, msgHead, message);
				dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
				dialog.setTitle(l.warning);
				dialog.setVisible(true);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return false;
		}
		
		AddEventToUndoStack(); //>>
		AddEventToConsole(); //>>
		System.out.println("AI2");
		InOut.inputsModified = true;
		
//		System.out.println("input accepted as " + input);
		return true;
	}
	

	void AddEventToUndoStack() {
	}

	void AddEventToConsole(){
	}

	public boolean RangeCheck(int val, int fieldNo){
		boolean mod = false;
		int maxInt[] = new int[4];
		int minInt[] = new int[4];
		maxInt[0]	= 20;			minInt[0] = 0;		// number of habitable rooms
		maxInt[1]	= 20;			minInt[1] = 1;		// number of bedrooms
		maxInt[2]	= 16;			minInt[2] = 0;		// number of occupants
		maxInt[3]   = 2050;			minInt[3] = 1950;		// boiler age
		if(val >= minInt[fieldNo] && val <= maxInt[fieldNo])mod = true;
		if(!mod)message = "Input is outside allowed range of " + minInt[fieldNo] + " to " + maxInt[fieldNo];
		return mod;
	}
	
	public boolean RangeCheck(double val, int fieldNo){
		if(fieldNo == -1)return true;
		boolean mod = false;
		double maxDouble[] = new double[46];
		double minDouble[] = new double[46];
		minDouble[0]	= 2;			maxDouble[0] = 6;		// room height (m)
		minDouble[1]	= 20;			maxDouble[1] = 500;		// floor area (m^2)
		minDouble[2]	= 100;			maxDouble[2] = 100000;	// radiator capacity (W)
		minDouble[3]	= 0.5;			maxDouble[3] = 2.0;	// radiator exponent (-)
		minDouble[4]	= 30;			maxDouble[4] = 120;		// radiator nominal suppply temperature (degC)
		minDouble[5]	= 20;			maxDouble[5] = 110;		// radiator nominal exit temperature (degC)
		minDouble[6]	= 10;			maxDouble[6] = 30;		// radiator nominal environment temperature (degC)
		minDouble[7]	= 0.01;	    	maxDouble[7] = 10.0;	// pump flow rate (kg/s)
		minDouble[8]	= 20;			maxDouble[8] = 1000;		// DHW tank capacity (lit)
		minDouble[9]	= -10;			maxDouble[9] = 10;		// design ambient temperature (degC)
		minDouble[10]	= 10;			maxDouble[10]= 30;		// set point temperature (degC)
		minDouble[11]	= 0.1;			maxDouble[11] = 6;		// set point temperature differential (degC)
		minDouble[12]	= 40;			maxDouble[12] = 80;		// DHW set point temperature (degC)
		minDouble[13]	= 0.1;			maxDouble[13] = 15;		// DHW temperature differential (degC)
		minDouble[14]	= 20;			maxDouble[14] = 50;		// minimum PIRC & TPM set point temperature (degC)
		minDouble[15]	= 40;			maxDouble[15] = 120;		// maximum PIRC & TPM set point temperature (degC)
		minDouble[16]	= 1;			maxDouble[16] = 10;		// PIRC & TPM proportional band (degC)
		minDouble[17]	= 200;			maxDouble[17] = 10000;	// PIRC & TPM integral action time (sec)
		minDouble[18]	= 0.10;			maxDouble[18] = 4.0;	// U Value
		minDouble[19]	= 1;			maxDouble[19] = 200;		// boiler firing rate (kW)
		minDouble[20]	= 0;			maxDouble[20] = 10;		// boiler lockout time (max)
		minDouble[21]	= 45;			maxDouble[21] = 120;		// upper boiler temperature limit (degC)
		minDouble[22]	= 5;			maxDouble[22] = 70;		// boiler lower limit of modulation (%)
		minDouble[23]	= 0.1;			maxDouble[23] = 10;		// boiler lower limit of modulation differential (%)
		minDouble[24]	= 0;			maxDouble[24] = 1000;	// boiler startup characteristics time (sec)
		minDouble[25]	= 1;			maxDouble[25] = 100;	// boiler startup characteristics rate (%)
		minDouble[26]	= 1;			maxDouble[26] = 50;   		// mass of boiler burner assembly (kg)
		minDouble[27]	= 0.1;			maxDouble[27] = 50;		// boiler jacket losses (W/K)
		minDouble[28]	= 1;			maxDouble[28] = 5;		// boiler DHW service capacity (kW)
		minDouble[29]	= 0.05;			maxDouble[29] = 0.8;	// thermostat mass (kg)
		minDouble[30]	= 0.01;			maxDouble[30] = 2.0;	// thermostat conductance (W/K)
		minDouble[31]	= 0.0;			maxDouble[31] = 10.0;	// thermostat heat anticipation (W)
		minDouble[32]	= 0.1;			maxDouble[32] = 3.0;	// TRV mass (kg)
		minDouble[33]	= 240;			maxDouble[33] = 1800;		// TPM cycle time (seconds)
		minDouble[34]	= 0;			maxDouble[34] = 300;		// TPM minimum ON time (seconds)
		minDouble[35]	= -20;			maxDouble[35] = 100;		// coeff CO2 and dTFlue ^0
		minDouble[36]	= -2;			maxDouble[36] = 2;		// coeff CO2 and dtFlue ^1,2,3
		minDouble[37]	= -20;			maxDouble[37] = 20;		// OTC alpha, beta and gamma
		minDouble[38]	= 0;			maxDouble[38] = 95;		// OTC set point temperatures
		minDouble[39]	= 5;			maxDouble[39] = 20000;	// radiator mass (kg)
		minDouble[40]	= 15;			maxDouble[39] = 95;	// radiator mass (kg)
//		minDouble[37]	= ;				maxDouble[37] = ;		//
//		minDouble[38]	= ;				maxDouble[38] = ;		//
//		minDouble[39]	= ;				maxDouble[39] = ;		//
//		minDouble[37]	= ;				maxDouble[37] = ;		//
//		minDouble[38]	= ;				maxDouble[38] = ;		//
//		minDouble[39]	= ;				maxDouble[39] = ;		//
		// may need to change the following line because of direct floating point number comparison
		if(val >= minDouble[fieldNo] && val <= maxDouble[fieldNo])mod = true;
		if(!mod)message = "Input is outside allowed range of " + minDouble[fieldNo] + " to " + maxDouble[fieldNo];
		return mod;
	}

	
}


