package uk.ac.strath.esru;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
//import java.lang.Object.*;
//import java.math.BigDecimal;
//import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.TimeUnit;

//import javax.swing.JDialog;
//import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.time.DateUtils;
import java.util.Arrays;

public class Algorithm {
	static boolean debug;
	static boolean debugStdOut;
	static int jj = 0; 
	final static int monsize = 288000; // monitoring file size
	final static int stepsize = 60 * 24; // Allow 1 minutely values to be recorded for each day
	
	// Monitored data read from mon file
	static double temp1[] = new double[monsize];
	static double temp2[] = new double[monsize];
	static double RH[] = new double[monsize];
	
	static String year[] = new String[monsize];
	static String month[] = new String[monsize];
	static String day[] = new String[monsize];
	static String hour[] = new String[monsize];
	static String minute[] = new String[monsize];
	static String second[] = new String[monsize];
	static Date date[] = new Date[monsize]; //Allow for 200 days post upgrade period
	
	static double tempRngMed1[] = new double[monsize];
	static double tempRngMed2[] = new double[monsize];
	static double RHRngMed[] = new double[monsize];
	static double DurationPostUpgradeMonPeriod;
	
	static int lineNoMonFilePostUpStart; // Evaluation environment algorithm after this array index (i.e. from start of post upgrade period)
	static int lineNoMonFilePostUpEnd; // Evaluation environment algorithm up to this array index (i.e. from start of post upgrade period)
	
	public String EnergyAlgorithm(){
		String ts, sts;
		String[] s 		= new String[30];
		String[] ss 		= new String[30];
		//If incomplete information has been entered then return with empty string
		s[0] = Language.BenchMSaving.trim();
		s[1] = Language.MeterMeasures.trim();
		s[2] = Language.PreUpStartMeterReading.trim();
		s[3] = Language.PreUpEndMeterReading.trim();
		s[4] = Language.PostUpStartMeterReading.trim();
		s[5] = Language.PostUpEndMeterReading.trim();
		s[6] = "unused";
		s[7] = "unused";
		s[8] = "unused";
		s[9] = "unused";
		s[10] = Language.PreUpStartDate.trim();
		s[11] = Language.PostUpStartDate.trim();
		s[12] = Language.PreUpEndDate.trim();
		s[13] = Language.PostUpEndDate.trim();
		s[14] = "unused";
		s[15] = "unused";
		
		ss[0] = "Benchmark saving";
		ss[1] = "Meter measures";
		ss[2] = "Pre-upgrade start meter reading";
		ss[3] = "Pre-upgrade end meter reading";
		ss[4] = "Post-upgrade start meter reading";
		ss[5] = "Post-upgrade end meter reading";
		ss[6] = "unused";
		ss[7] = "unused";
		ss[8] = "unused";
		ss[9] = "unused";
		ss[10] = "Pre-upgrade start date";
		ss[11] = "Post-upgrade start date";
		ss[12] = "Pre-upgrade end date";
		ss[13] = "Post-upgrade end date";
		ss[14] = "unused";
		ss[15] = "unused";
		
		for(int i = 1; i <= 15; i++){
			if(s[i].equals("undefined") || s[i].equals("")){
				InOut.UserMsg("incomplete data: " + s[i] + " is not defined (item #" + i + ")");
				return "";
			}
		}
		
		
		
		// Energy normalisation procedures to be applied to the 
		// Post Operations Evaluation Tool.


		// DaysNonHeatingSeason   = Number of days in the non-heating season (days)
		// StartDatePreUpgradeMonPeriod  = Start date of pre-upgrade monitoring period
		// EndDatePreUpgradeMonPeriod  = End date of pre-upgrade monitoring period
		// StartDatePostUpgradeMonPeriod  = Start date of post-upgrade monitoring period
		// EndDatePostUpgradeMonPeriod  = End date of post-upgrade monitoring period
		// DurationPreUpgradeMonPeriod  = Duration of the pre-upgrade monitoring period (days)
		// DurationPostUpgradeMonPeriod  = Duration of the post-upgrade monitoring period (days)

		// GMRSTD   = Gas meter reading recorded at start date
		// GMREND   = Gas meter reading recorded at end date

		// EnergyUsedPreUpgradePeriod   = Total energy consumption during the pre-upgrade period (kWh)
		// EnergyUsedPostUpgradePeriod   = Total energy consumption during the post-upgrade period (kWh)
		// DHWNHS   = Domestic hot water consumed during the non-heating season (kWh)
		// DHWPreUpgradePeriod   = Sum of domestic hot water use pre-upgrade (kWh)
		// DHWPostUpgradePeriod   = Sum of domestic hot water use post-upgrade (kWh)
		// DELESD   = Sum of domestic electrical appliance use pre-upgrade (kWh)
		// DELEED   = Sum of domestic electrical appliance use post-upgrade (kWh)

		// SpaceHeatOnlyEnergyPreUpgrade   = Energy consumed for space heating only pre-upgrade (kWh)
		// SpaceHeatOnlyEnergyPostUpgrade   = Energy consumed for space heating only post-upgrade (kWh)

		// MeanHourlyExtTempPreUp  = Mean hourly external temperature during the pre-upgrade period (癈)
		// MeanHourlyExtTempPostUp  = Mean hourly external temperature during the post-upgrade period (癈)
		// MeanIntTempPreUp   = Mean indoor temperature during the pre-upgrade period (癈)
		// MeanIntTempPostUp   = Mean indoor temperature during the post-upgrade period (癈)

		// MeanIntTempDegreeDayCalc   = Mean indoor temperature assumed in standard degree day calculation (癈)
		// DegreeDayBaseTemp   = Base temperature taken from standard heating degree days calculation (癈)

		// HeatingDegreeHrsPreUp   = Heating degree hours for the pre-upgrade period
		// HeatingDegreeHrsPostUp   = Heating degree hours for the post-upgrade period
		// HeatingDegreeDaysPreUp  = Heating degree days calculated for the pre-upgrade period 
		// HeatingDegreeDaysPostUp  = Heating degree days calculated for the post-upgrade period
		// HeatingDegreeDaysCorrPreUp  = Heating degree days accounting for outdoor climate differences and impact of insulation (pre-upgrade)
		// HeatingDegreeDaysCorrPostUp  = Heating degree days accounting for outdoor climate differences and impact of insulation (post-upgrade) 

		// NormEnergyUsePreUp   = Normalised energy consumption for pre-upgrade period
		// PercDiffPrePostEnergy   = Percentage difference between pre- and post-upgrade energy consumption (%)
		// PercExpectedBenchmarkSaving   = Expected benchmark energy saving (%)
		// PercDiffExpectedAndActualSavings   = Difference between PercDiffPrePostEnergy and PercExpectedBenchmarkSaving , this is the OUTPUT

      double GasConversionFactor,EnergyUsedPreUpgradePeriod,EnergyUsedPostUpgradePeriod,DELPRE,DELPST,SpaceHeatOnlyEnergyPreUpgrade,SpaceHeatOnlyEnergyPostUpgrade,
      MeanIntTempDegreeDayCalc,DegreeDayBaseTemp,MeanIntTempPreUp,MeanIntTempPostUp,NormEnergyUsePreUp,PercDiffPrePostEnergy,
      PercExpectedBenchmarkSaving,PercDiffExpectedAndActualSavings,DHWEnergyPerDay,ApplianceEnergyPerDay,DHWPreUpgradePeriodGas,
      DHWPostUpgradePeriodGas,AppliancePreUpgradePeriodElec,AppliancePostUpgradePeriodElec,
      PerExpVsBM;
      int DurationPreUpgradeMonPeriod,HeatingDegreeHrsPreUp,HeatingDegreeHrsPostUp,HeatingDegreeDaysPreUp,
      HeatingDegreeDaysPostUp,HeatingDegreeDaysCorrPreUp,HeatingDegreeDaysCorrPostUp,IMRDGAS,IMRDELE;  
		      
      GasConversionFactor=0;
      MeanIntTempDegreeDayCalc=19.5;
      DegreeDayBaseTemp=15.5;
		      
		// Mean energy content for DHW (DHWEnergyPerDay) kWh/ day
		// (Reference: Measurement of Domesti// Hot Water consumption in 
		// Dwellings, Energy Saving Trust, 2008)

		      DHWEnergyPerDay=4.7;

		// The annual electricity consumption related to lighting and 
		// appliance use per person can be calculated as follows 
		// (Reference: Electricity supply, electricity supplied (net), 
		// electricity available, electricity consumption and electricity 
		// sales (DUKES 5.4); Electricity: chapter 5, Digest of United 
		// Kingdom Energy Statistics (DUKES)):
		 
		// Annual electricity consumption (kWh) for cooking,water heating, 
		// lighting and appliance use (2013) = 8.846x10**10


		// Population of the UK                  = 64.1x10**6


		// Electricity consumption for lighting and appliances (ApplianceEnergyPerDay)
		// (annual kWh/ person):


		      ApplianceEnergyPerDay=9.8;



		//***************************************************************
		// STEP 1
		// Number of days elapsed between start and end dates of 
		// each monitoring period.
		//***************************************************************
		 
		// Determines the number of days elapased between the start and 
		// end dates of both pre- and post-upgrade monitoring periods 
		// (each period must be of equal length as this information will 
		// be carried forward to STEP 5).

		// The user will enter the start and end dates of each
		// monitoring period and the difference will be calculated as
		// follows, with the result provided in days.


		      //SimpleDateFormat sdf = new SimpleDateFormat("EEE MMM dd kk:mm:ss z yyyy");
		      SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
		      DurationPreUpgradeMonPeriod=0; 
		      DurationPostUpgradeMonPeriod=0; 
		      try{
//		    	  Calendar c = Calendar.getInstance();
		    	  Date d1 = sdf.parse(Language.PreUpStartDate);
		    	  Date d2 = sdf.parse(Language.PreUpEndDate);
		    	  DurationPreUpgradeMonPeriod = (int)getDateDiff(d1, d2, TimeUnit.DAYS);
		    	  
		    	  d1 = sdf.parse(Language.PostUpStartDate);
		    	  d2 = sdf.parse(Language.PostUpEndDate);
		    	  DurationPostUpgradeMonPeriod = (int)getDateDiff(d1, d2, TimeUnit.DAYS);

		    	  if(debugStdOut)System.out.println("DurationPreUpgradeMonPeriod = " + DurationPreUpgradeMonPeriod);
		    	  if(debugStdOut)System.out.println("DurationPostUpgradeMonPeriod = " + DurationPostUpgradeMonPeriod);
//			      c.setTime(d);
//			      long time = c.getTimeInMillis();
			      
//			      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
//			      System.out.println(df.format(d));
		    	  
		    	  Language.DebugDurationPostUpgradeMonPeriod = Double.toString(DurationPostUpgradeMonPeriod);
		    	  Language.DebugDurationPreUpgradeMonPeriod = Double.toString(DurationPreUpgradeMonPeriod);
		      }catch(ParseException pe){
		    	  // 
		      }



		//***************************************************************
		// STEP 2 
		// Total energy consumption pre- and post-upgrade periods.
		//***************************************************************

		// Calculate total gas and electricity consumption over  
		// monitoring periods. Convert unit of gas meter readings to kWh 

		// The following calculations are done for the pre- and post-upgrade periods.


	      if(Language.MeterMeasures.trim().equals("kWh"))GasConversionFactor=1;
	      if(Language.MeterMeasures.trim().equals("cubic meters"))GasConversionFactor=11.14264;
	      if(Language.MeterMeasures.trim().equals("cubic feet"))GasConversionFactor=31.53367;
	      EnergyUsedPreUpgradePeriod=(Double.parseDouble(Language.PreUpEndMeterReading) - Double.parseDouble(Language.PreUpStartMeterReading))*GasConversionFactor;
	      EnergyUsedPostUpgradePeriod=(Double.parseDouble(Language.PostUpEndMeterReading) - Double.parseDouble(Language.PostUpStartMeterReading))*GasConversionFactor;
	      if(debugStdOut)System.out.println("EnergyUsedPreUpgradePeriod = " + EnergyUsedPreUpgradePeriod);
	      if(debugStdOut)System.out.println("EnergyUsedPostUpgradePeriod = " + EnergyUsedPostUpgradePeriod);
	      Language.DebugEnergyUsedPostUpgradePeriod = Double.toString(EnergyUsedPostUpgradePeriod);
		  Language.DebugEnergyUsedPreUpgradePeriod = Double.toString(EnergyUsedPreUpgradePeriod);

		// For electrically heated dwellings there may be more than one 
		// meter reading. If this is the case, the user should enter the
		// sum of all values (which will already have the units kWh).


		//***************************************************************
		// STEP 3
		// Energy consumption related to DHW, lighting and appliances.
		//***************************************************************

		// Calculate the gas/ electricity consumption related to DHW and
		// electrical appliance use.

		/////////////////////////////IGNORE//////////////////////////////
		// If the monitoring period includes the non-heating season, it
		// may be possible to calculate a daily consumption value for DHW
		// by dividing the recorded gas consumption data from the 
		// non-heating season by the monitoring period (days) calculated
		// in STEP 1. The assumption for this calculation is that no heating
		// is used during the non-heating season.

		// DHWNHS=(GMRSTD*(CNVFG1 OR CNVFG2)-GMREND*(CNVFG1 OR CNVFG2))/DaysNonHeatingSeason
		/////////////////////////////IGNORE//////////////////////////////

		// The default values listed above for DHW and lighting and 
		// electrical appliance use can be applied in the following 
		// calculations:


		// Pre-upgrade period:

		      DHWPreUpgradePeriodGas=DurationPreUpgradeMonPeriod*DHWEnergyPerDay;
		      Language.DebugDHWPreUpgradePeriodGas = Double.toString(DHWPreUpgradePeriodGas);
		      if(debugStdOut)System.out.println("DHWPreUpgradePeriodGas = " + DHWPreUpgradePeriodGas);

		// Post-upgrade period:

		      DHWPostUpgradePeriodGas=DurationPostUpgradeMonPeriod*DHWEnergyPerDay;
		      Language.DebugDHWPostUpgradePeriodGas = Double.toString(DHWPostUpgradePeriodGas);
		      if(debugStdOut)System.out.println(" DHWPostUpgradePeriodGas = " + DHWPostUpgradePeriodGas);

		// To calculate the electricity consumption related to lighting
		// and appliance use during the pre- and post-upgrade periods, the
		// following calculations are applied:

		// Pre-upgrade period:

		      AppliancePreUpgradePeriodElec=(DurationPreUpgradeMonPeriod)*ApplianceEnergyPerDay;
		      Language.DebugAppliancePreUpgradePeriodElec = Double.toString(AppliancePreUpgradePeriodElec);
		      if(debugStdOut)System.out.println("AppliancePreUpgradePeriodElec = " + AppliancePreUpgradePeriodElec);
		    		  
		// Post-upgrade period:

		      AppliancePostUpgradePeriodElec=(DurationPostUpgradeMonPeriod)*ApplianceEnergyPerDay;
		      Language.DebugAppliancePostUpgradePeriodElec = Double.toString(AppliancePostUpgradePeriodElec);
		      if(debugStdOut)System.out.println("AppliancePostUpgradePeriodElec = " + AppliancePostUpgradePeriodElec);

		//***************************************************************
		// STEP 4
		// Energy consumption related to space heating.
		//***************************************************************

		// The energy consumption (kWh) related to space heating 
		// only for both pre- and post-upgrade periods can now be 
		// determined.
		      SpaceHeatOnlyEnergyPreUpgrade = 0; // init
		      SpaceHeatOnlyEnergyPostUpgrade = 0;
		      if(Language.MeterMeasures.trim().equals("cubic feet") || Language.MeterMeasures.trim().equals("cubic meters")){
		    	  // For gas centrally heated dwellings:
		    	  SpaceHeatOnlyEnergyPreUpgrade = EnergyUsedPreUpgradePeriod - DHWPreUpgradePeriodGas;
		    	  SpaceHeatOnlyEnergyPostUpgrade = EnergyUsedPostUpgradePeriod - DHWPostUpgradePeriodGas;
		      }else if(Language.MeterMeasures.trim().equals("kWh")){
		    	  // For electrically heated dwellings:
		    	  SpaceHeatOnlyEnergyPreUpgrade = EnergyUsedPreUpgradePeriod - AppliancePreUpgradePeriodElec - DHWPreUpgradePeriodGas;
		    	  SpaceHeatOnlyEnergyPostUpgrade = EnergyUsedPostUpgradePeriod - AppliancePostUpgradePeriodElec - DHWPostUpgradePeriodGas;
		      }else{
		    	  InOut.UserMsg("Error: Heating fuel not defined, aborting calculation");
		      }
		      Language.DebugSpaceHeatOnlyEnergyPreUpgrade = Double.toString(SpaceHeatOnlyEnergyPreUpgrade);
		      Language.DebugSpaceHeatOnlyEnergyPostUpgrade = Double.toString(SpaceHeatOnlyEnergyPostUpgrade);
		      if(debugStdOut)System.out.println("SpaceHeatOnlyEnergyPreUpgrade = " + SpaceHeatOnlyEnergyPreUpgrade);
		      if(debugStdOut)System.out.println("SpaceHeatOnlyEnergyPostUpgrade = " + SpaceHeatOnlyEnergyPostUpgrade);
		      


		//***************************************************************
		// STEP 5
		// Normalisation of pre-upgrade space heating energy consumption.
		//***************************************************************
		// MeanHourlyExtTempPreUp  = Mean hourly external temperature during the 
		//            pre-upgrade period (癈)

		// MeanHourlyExtTempPostUp  = Mean hourly external temperature during the 
		//            post-upgrade period (癈)

		// MeanIntTempPreUp   = Mean indoor temperature during the pre-upgrade period (癈)
		// MeanIntTempPostUp   = Mean indoor temperature during the post-upgrade period (癈)

		// MeanIntTempDegreeDayCalc   = Mean indoor temperature assumed in standard degree 
		//            day calculation (19.5癈)
		// DegreeDayBaseTemp   = Base temperature taken from standard heating degree 
		//            days calculation (15.5癈)

		// HeatingDegreeHrsPreUp   = Heating degree hours for the pre-upgrade period
		// HeatingDegreeHrsPostUp   = Heating degree hours for the post-upgrade period
		// HeatingDegreeDaysPreUp  = Heating degree days calculated for the pre-upgrade period 
		// HeatingDegreeDaysPostUp  = Heating degree days calculated for the post-upgrade period
		// HeatingDegreeDaysCorrPreUp  = Heating degree days accounting for outdoor climate 
		//            differences and impact of insulation (pre-upgrade)
		// HeatingDegreeDaysCorrPostUp  = Heating degree days accounting for outdoor climate 
		//            differences and impact of insulation (post-upgrade)


		// Two approaches are being proposed for the normalisation of 
		// space heating energy consumption. These are described below.

		// For the purposes of this procedure, a standard value for
		// the base temperature has been taken and defined as
		// DegreeDayBaseTemp = 15.5癈. The standard mean indoor temperature of
		// 19.5�// (used in the standard HDD method) has also been applied 
		// in the second approach.

		// APPROACH #1
		// This involves normalising the space heating energy 
		// consumption (calculated in STEP 4) in order to account for 
		// weather differences between the pre- and post-upgrade 
		// monitoring periods. This is done through the use of heating
		// degree days (HDD). 


		// The basi// formula for calculating HDD is as follows:

		// HDD = Base temperature - Outdoor temperature

		// Over the period specified in STEP 1, the mean outdoor 
		// temperature is calculated and subtracted from the base 
		// temperature on an hourly basis. The HDD value is then 
		// calculated by dividing the sum of the heating degree hours 
		// by 24.

		// The following equations are used to calculate the heating
		// degree hours for the pre- and post-upgrade periods.


	      // Get degree days
	      HeatingDegreeDaysPostUp = 0;
	      HeatingDegreeDaysPreUp = GetDegreeDays(Language.PreUpStartDate, Language.PreUpEndDate, DegreeDayBaseTemp);
//	      System.out.println("Language.PreUpStartDate = " + Language.PreUpStartDate);
//	      System.out.println("Language.PreUpEndDate   = " + Language.PreUpEndDate);
//	      System.out.println("DegreeDayBaseTemp       = " + DegreeDayBaseTemp);
//	      System.out.println("Language.WeatherFile    = " + Language.WeatherFile);
	      if(debugStdOut)System.out.println("HeatingDegreeDaysPreUp = " + HeatingDegreeDaysPreUp);
	      if(HeatingDegreeDaysPreUp != -999){
	    	  HeatingDegreeDaysPostUp = GetDegreeDays(Language.PostUpStartDate, Language.PostUpEndDate, DegreeDayBaseTemp);
	    	  if(debugStdOut)System.out.println("HeatingDegreeDaysPostUp = " + HeatingDegreeDaysPostUp);
	    	  
	      }else{
//	    	  InOut.UserMsg("Error: In return from method GetdegreeDays in class EnergyAlgorithm, aborting calculation");
	    	  Language.Rating = "undefined";
	    	  return "-999";
	      }
	      Language.DebugHeatingDegreeDaysPreUp = Double.toString(HeatingDegreeDaysPreUp);
	      Language.DebugHeatingDegreeDaysPostUp = Double.toString(HeatingDegreeDaysPostUp);
		// The pre-upgrade space heating energy consumption (SpaceHeatOnlyEnergyPreUpgrade) 
		// can now be normalised.


		      NormEnergyUsePreUp=(SpaceHeatOnlyEnergyPreUpgrade/HeatingDegreeDaysPreUp)*HeatingDegreeDaysPostUp;
		      Language.DebugNormEnergyUsePreUp = Double.toString(NormEnergyUsePreUp);
		      if(debugStdOut)System.out.println("NormEnergyUsePreUp = " + NormEnergyUsePreUp);

		// APPROACH #2
		// This involves normalising the space heating energy 
		// consumption (calculated in STEP 4) in order to account for 
		// weather differences and the impact of the improved heat 
		// transfer performance of the building envelope fabric
		// between the pre- and post-upgrade monitoring periods. 
		// This is done through the use of an adjusted heating degree 
		// days (HDD) method.

		// The average indoor and outdoor measured temperatures are
		// calculated on an hourly basis for the defined pre- and 
		// post-upgrade period. These are then substituted into the 
		// formula shown below, in order to calculate the adjusted HDD.


//		      HeatingDegreeHrsPreUp=((DegreeDayBaseTemp+(MeanIntTempPreUp-MeanIntTempDegreeDayCalc))-MeanHourlyExtTempPreUp
//		      HeatingDegreeHrsPostUp=((DegreeDayBaseTemp+(MeanIntTempPostUp-MeanIntTempDegreeDayCalc))-MeanHourlyExtTempPostUp
//
//		      HeatingDegreeDaysCorrPreUp=SUM(HeatingDegreeHrsPreUp)/24
//		      HeatingDegreeDaysCorrPostUp=SUM(HeatingDegreeHrsPostUp)/24





		//***************************************************************
		// STEP 6
		// Comparison of results with expected outcomes.
		//***************************************************************

		// The initial part of this step involves calculating the 
		// percentage difference between the normalised pre-upgrade 
		// space heating energy consumption and the post-upgrade space 
		// heating energy consumption (PercDiffPrePostEnergy).


		      PercDiffPrePostEnergy=100*((NormEnergyUsePreUp-SpaceHeatOnlyEnergyPostUpgrade)/NormEnergyUsePreUp);      
		      Language.DebugPercDiffPrePostEnergy = Double.toString(PercDiffPrePostEnergy);
		      
		// A comparison is then made between the value for PercDiffPrePostEnergy and an 
		// expected benchmark percentage energy saving (PercExpectedBenchmarkSaving) for the 
		// type of dwelling being assessed.

		      getBenchmark();
		      if(Language.BenchMSaving.trim().equals("undefined")){
		    	  InOut.UserMsg("Warning: Benchmark undefined");
		    	  Language.Rating = "undefined";
		    	  //return "-999";
		      }else{
		    	  
			      PercExpectedBenchmarkSaving = Double.parseDouble(Language.BenchMSaving);
			      PercDiffExpectedAndActualSavings = PercExpectedBenchmarkSaving - PercDiffPrePostEnergy; // Which is the output and should be present on the sign off certificate
			      if(debugStdOut)System.out.println("PercDiffExpectedAndActualSavings = " + PercDiffExpectedAndActualSavings);
			      PerExpVsBM = PercDiffPrePostEnergy / PercExpectedBenchmarkSaving * 100.;
			      
			      ts = Double.toString(PerExpVsBM);
			      sts = ts.substring(0, Math.min(ts.length(), 4)); // first four characters 
			      //EnergyOutcome = "Actual savings " + sts + "% of benchmark";
			      int iii = (int) PerExpVsBM;
			      Language.Rating = Integer.toString(iii);
			      
			      if(HeatingDegreeDaysPreUp == -999 || HeatingDegreeDaysPostUp == -999){
				    Language.Rating = "undefined";
				    return "-999";
			      }
		      }
		      return Double.toString(PercDiffPrePostEnergy);
	}
	
	public static int GetDegreeDays(String startDate, String endDate, Double BaseTemp){
		
		String poetWFile = InOut.weatherFileDir + File.separator + "Weather_" + Language.WeatherFile + "POET.csv";
		double degHours = 0;
		Date curDate;

		try {
			// Get monitoring period start and end dates
			SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
			Date monStart 	= sdf.parse(startDate);
			Date monEnd		= sdf.parse(endDate);

			// Get one day before monStart and one day after monEnd to aid comparison later on
			Calendar cal = Calendar.getInstance();

// 			No need to take one day before monStart because the meter reading was taken at that time and 
// 			we start the energy calculation the day after then meter reading
//		    cal.setTime(monStart);
//		    cal.add(Calendar.DAY_OF_MONTH, -1);
//		    monStart = cal.getTime();
		    
		    cal.setTime(monEnd);
		    cal.add(Calendar.DAY_OF_MONTH, 1);
		    monEnd = cal.getTime();
		    
		    // Define local variables
		    int hour = 0; int i = 0; int ii = 1;
		    String tmpStr = "0";
		    double avgTemp = 0;
		    double totTemp = 0;
		    boolean monDatesBfWeather = true;
		    boolean monDatesAfWeather = true;
		    boolean computeDD = false;
		    
			// Read poet weather file
		    File f = new File(poetWFile);
		    if(!f.exists()){
		    	InOut.UserMsg("Error: Weather file does not exist");
		    	return -999;
		    }
if(jj<Language.NN)Language.DebugExtended[jj] = "Selected weather file " + poetWFile + "\n"; jj++;
if(jj<Language.NN)Language.DebugExtended[jj] = "Initial degree hours " + degHours + "\n"; jj++;
			FileReader fr = new FileReader(poetWFile);
			BufferedReader br = new BufferedReader(fr);
			String line;
			SimpleDateFormat sdfDate = new SimpleDateFormat("dd/MM/yyyy");
			SimpleDateFormat sdfDateTime = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
			SimpleDateFormat sdfHour = new SimpleDateFormat("HH");
			curDate = sdfDate.parse("01/12/2000"); // Initialise to date in past 

			//System.out.println(sdfDate.format(monStart));
			//System.out.println(sdfDate.format(monEnd));
			while ((line = br.readLine()) != null) {
				String wData[] 	= line.split(",");
				if(!curDate.equals(sdfDate.parse(wData[0]))){
					curDate = sdfDate.parse(wData[0]);
					if(jj<Language.NN)Language.DebugExtended[jj] =  "\n"; jj++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "reading date = " + wData[0] + "\n"; jj++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "T_avg , degHours \n"; jj++;

				}
				Date weatherDate 		= sdfDate.parse(wData[0]);
				if(monDatesBfWeather){
					if(weatherDate.before(monStart) || weatherDate.equals(monStart))monDatesBfWeather = false; //
					computeDD = true;
				}
				if(monDatesAfWeather){
					if(weatherDate.after(monEnd) || weatherDate.equals(monEnd))monDatesAfWeather = false;
				}

				if(computeDD){
					if(weatherDate.after(monStart) && weatherDate.before(monEnd)){
						
						// Get hour
						Date dt = sdfDateTime.parse(wData[0] + " " + wData[1]);
						tmpStr = sdfHour.format(dt);
						if (hour != (int) Double.parseDouble(tmpStr)){ // new hour
							if( i != 0 ){
								avgTemp = totTemp / i; 
								if((BaseTemp - avgTemp) > 0){
									degHours += BaseTemp - avgTemp; 
								}

								if(jj<Language.NN)Language.DebugExtended[jj] = avgTemp + " , " + degHours + "\n"; jj++;
//								if(jj<Language.NN)Language.DebugExtended[jj] = "Average temperature  = " + avgTemp + "\n"; jj++;
//								if(jj<Language.NN)Language.DebugExtended[jj] = "Degree hours         = " + degHours + "\n"; jj++;
								//if(jj<Language.NN)Language.DebugExtended[jj] = "Hour degH*24 T_avg T_tot\n"; jj++;
							}
							totTemp = 0;
							i = 0;
						}else{
							
						}
						hour = (int) Double.parseDouble(tmpStr);
						i++; 
						totTemp += Double.parseDouble(wData[2]);
						//if(jj<Language.NN)Language.DebugExtended[jj] = tmpStr + "   " + degHours + "  " + totTemp + "\n"; jj++;
					}
				}
			}
			br.close();
			fr.close();

			if(monDatesBfWeather){
				InOut.UserMsg("Error: Meters read before weather data monitoring commences");
				return -999;
			}
			if(monDatesAfWeather){
				InOut.UserMsg("Error: Meters read after weather data monitoring finishes");
				return -999;
			}
		} catch (ParseException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return (int) (degHours / 24);
	}
	
	public static long getDateDiff(Date date1, Date date2, TimeUnit timeUnit) {
	    double L1 = date1.getTime();
	    double L2 = date2.getTime();
	    double Diff = L2 - L1;
	    float output = new Float(Math.round(Diff / 86400000));
	    return (long) output;

	    // The following code gives an error of one day if run across 
	    // a period spanning when time changes for daylight savings
	    //long diffInMillies = Math.round(date2.getTime() - date1.getTime());
	    //return timeUnit.convert(diffInMillies,TimeUnit.MILLISECONDS);
	    
	}

	public static boolean getBenchmark(){
		boolean found = false;
		//<< John Allison to advise how bmc is dependent on tfa
		if(!Language.ArchiType.equals("undefined")){
			if(!Language.Construction.equals("undefined")){
				if(!Language.Occupants.equals("undefined")){
					if(!Language.TFA.equals("undefined")){
						
						// Benchmark is only defined if secondary upgrade measures are not present
						if(Language.UpgradeMeasures1.equals("undefined") && Language.UpgradeMeasures2.equals("undefined")){
							//System.out.println("calling bm");
							InOut.openPOETBuildingsDB("get BM");
							//System.out.println("af bm " + Language.BenchMSaving);
							found = true;
						}
					}
				}
			}
		}
		if(!found)Language.BenchMSaving = "undefined";
		return found;
	}

	public static void EnvAlgorithm(){
		// Make sure mon file is read before this method is called
		if(Language.monDataFile.trim().equals("undefined")){
			InOut.UserMsg("Error: Monitored data file has not been loaded");
			return;
		}else{
			InOut.readMonDataFile("no graph"); // we do not need to create graph file here
		}
		String degC = String.valueOf(Character.toChars(186)) + "C";
		String degCs = degC + " ";
		
		int[] T1 = getStats("T1");
		int[] T2 = getStats("T2");
		int[] RH = getStats("RH");
		System.out.println("");
			
		Language.DebugT1Mean = Integer.toString(T1[0]);
		Language.DebugT1MoreThanHigher = Integer.toString(T1[1]);
		Language.DebugT1MoreThanHigh = Integer.toString(T1[2]);
		Language.DebugT1LessThanLow = Integer.toString(T1[3]);
		Language.DebugT1LessThanLower = Integer.toString(T1[4]);

		Language.DebugT2Mean = Integer.toString(T2[0]);
		Language.DebugT2MoreThanHigher = Integer.toString(T2[1]);
		Language.DebugT2MoreThanHigh = Integer.toString(T2[2]);
		Language.DebugT2LessThanLow = Integer.toString(T2[3]);
		Language.DebugT2LessThanLower = Integer.toString(T2[4]);
		
		Language.DebugRHMean = Integer.toString(RH[0]);
		Language.DebugRHMoreThanHigher = Integer.toString(RH[1]);
		Language.DebugRHMoreThanHigh = Integer.toString(RH[2]);
		Language.DebugRHLessThanLow = Integer.toString(RH[3]);
		Language.DebugRHLessThanLower = Integer.toString(RH[4]);
		
		
		if(T1[5] == 0 && T2[5] == 0 && RH[5] == 0){
			Language.envOutcome = "Temperatures and humidity levels are satisfactory";
		}else if(T1[5] == 0 && T2[5] == 0){
			Language.envOutcome = "Temperatures satisfactory but humidity levels are not (details in report)";
		}else if(RH[5] == 0){
			Language.envOutcome = "Humidity levels satisfactory but temperatures are not (details in report)";
		}else{
			Language.envOutcome = "Temperatures and humidity levels are not satisfactory (details in report)";
		}
		Language.res1 = "Median temperature 1 is higher than alarm limit for " + T1[1] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res2 = "Median temperature 1 is higher than alert limit for " + T1[2] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res3 = "Median temperature 1 is lower than alert limit for " + T1[3] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res4 = "Median temperature 1 is lower than alarm limit for " + T1[4] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		
		Language.res5 = "Median temperature 2 is higher than alarm limit for " + T2[1] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res6 = "Median temperature 2 is higher than alert limit for " + T2[2] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res7 = "Median temperature 2 is lower than alert limit for " + T2[3] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res8 = "Median temperature 2 is lower than alarm limit for " + T2[4] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		
		Language.res9 = "Median relative humidity is higher than alarm limit for " + RH[2] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";
		Language.res10 = "Median relative humidity is lower than alarm limit for " + RH[3] + 
						" days out of " + Algorithm.DurationPostUpgradeMonPeriod + " monitored days.\n";

		Language.res1 = "Temperatures 1 and 2 have means of " + SigFig(T1[0]) + " and " + SigFig(T2[0]) + degC + 
						" respectively.\nMean RH is " + RH[0] + " %.";
		Language.res2 = "Temperatures 1 and 2 are above " + Language.stptTemp1 + degCs + 
						SigFig(T1[1]) + " and " + SigFig(T2[1]) + " % of time respectively.\n";
		Language.res3 = "They are above " + Language.stptTemp2 + degCs + SigFig(T1[2]) + " and " + SigFig(T2[2]) + " % of time.\n";
		Language.res4 = "They are below " + Language.stptTemp4 + degCs + SigFig(T1[3]) + " and " + SigFig(T2[3]) + " % of time.\n";
		Language.res5 = "They are below " + Language.stptTemp3 + degCs + SigFig(T1[4]) + " and " + T2[4] + " % of time.\n";
		Language.res6 = "Relative humidity is above " + Language.stptRH1 + "%, " + RH[2] + " % of time.\n";
		Language.res7 = "It is below " + Language.stptRH2 + "%, " + RH[3] + " % of time.";

		if(debugStdOut){
/*			System.out.println(Language.res1);
			System.out.println(Language.res2);
			System.out.println(Language.res3);
			System.out.println(Language.res4);
			System.out.println(Language.res5);
			System.out.println(Language.res6);
			System.out.println(Language.res7);
			*/
//			System.out.println(RH[1] + " " + RH[2] + " " + RH[3] + " " + RH[4] + " ");
			try {
				InOut.savePOETDebugFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

	}

	
	// This method returns the number of times the median of an array is outwith a range 
	// This median is worked out for each day beginning at midnight
	public static int[] getStats(String data){
		double curNum[] = new double[stepsize];
		double readVal = 0;
		double rngMedian = 0;
		double higher = 0;
		double high = 0;
		double lower = 0;
		double low = 0;
		double rngHigh = 0;
		double rngLow = 0;
		Date curDate = Algorithm.date[lineNoMonFilePostUpStart + 1];
		Date prevDate = Algorithm.date[lineNoMonFilePostUpStart + 1];
		Calendar cal1 = Calendar.getInstance();
		Calendar cal2 = Calendar.getInstance();
		boolean nextDay = false;
		boolean sameDay = false;
		int j = 0;
		
		// Set limits
		if(data.trim().equals("T1") || data.trim().equals("T2")){
			higher = Double.parseDouble(Language.stptTemp1);
			high = Double.parseDouble(Language.stptTemp2);
			low = Double.parseDouble(Language.stptTemp4);
			lower = Double.parseDouble(Language.stptTemp3);
		}else if(data.trim().equals("RH")){
			higher = 100;
			high = Double.parseDouble(Language.stptRH1);
			lower = 0;
			low = Double.parseDouble(Language.stptRH2);
		}


		double sum = 0;
		for(int i = 0; i < stepsize; i++)curNum[i] = 0;
		int Stats[] = new int[]{0, 0, 0, 0, 0, 0, 0, 0}; 
		// This array element 	0 = mean 
		//						number of times median is 
		//						1 = more than higher  
		//						2 = more than high
		//						3 = less than low
		//						4 = less than lower
		//						5 = sum of all alerts and alarms
		//						6 = median
		//						7 = unused

		//System.out.println("start at " + lineNoMonFilePostUpStart + " end at " + lineNoMonFilePostUpEnd);
		// Start reading from start of post upgrade period executes for the last monitored day as well
		for(int i = lineNoMonFilePostUpStart + 1; i <= lineNoMonFilePostUpEnd ; i++){ 
			curDate = Algorithm.date[i];
//			System.out.println("Line read with date = " + curDate.toString());
			
			cal1.setTime(curDate);
			cal2.setTime(prevDate);
			sameDay = cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
			          cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR);
			nextDay = !sameDay;
			
			if(nextDay || i == lineNoMonFilePostUpEnd){ // if next day or the last line to be read (because "next day" is not set for the last day)  
				rngMedian = getMedian(curNum, j - 1);
//				System.out.println("number of data items = " + j);
				Stats[6] = (int) (rngMedian * 100);

				if(rngMedian > higher){
					Stats[1]++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "current date | hour | min | sec = " + curDate.toString() + "|" + 
							Algorithm.hour[i] + "|" + 
							Algorithm.minute[i] + "|" + 
							Algorithm.second[i] + "\n"; jj++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "Median > higher limit = " + rngMedian + ">" + higher  +"\n"; jj++;

				}else if(rngMedian > high){
					Stats[2]++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "current date | hour | min | sec = " + curDate.toString() + "|" + 
							Algorithm.hour[i] + "|" + 
							Algorithm.minute[i] + "|" + 
							Algorithm.second[i] + "\n"; jj++;
							if(jj<Language.NN)Language.DebugExtended[jj] = "Median > high limit = " + rngMedian + ">" + high  +"\n"; jj++;
				}else if(rngMedian < low){
					Stats[3]++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "current date | hour | min | sec = " + curDate.toString() + "|" + 
							Algorithm.hour[i] + "|" + 
							Algorithm.minute[i] + "|" + 
							Algorithm.second[i] + "\n"; jj++;
							if(jj<Language.NN)Language.DebugExtended[jj] = "Median < low limit = " + rngMedian + "<" + low  +"\n"; jj++;
				}else if(rngMedian < lower){
					Stats[4]++;
					if(jj<Language.NN)Language.DebugExtended[jj] = "current date | hour | min | sec = " + curDate.toString() + "|" + 
							Algorithm.hour[i] + "|" + 
							Algorithm.minute[i] + "|" + 
							Algorithm.second[i] + "\n"; jj++;
							if(jj<Language.NN)Language.DebugExtended[jj] = "Median < lower limit= " + rngMedian + "<" + lower  +"\n"; jj++;
				}
//				System.out.println("Median for " + prevDate.toString() + " = " + rngMedian);
//				if(nextDay)System.out.println(prevDate.toString() + " is after " + curDate.toString());
				sum = 0;
				j = 0;
			}
			
			if(data.equals("T1")){
				readVal = temp1[i];
			}else if(data.equals("T2")){
				readVal = temp2[i]; // add next value
			}else if(data.equals("RH")){
				readVal = RH[i]; // add next value
			}
			sum += readVal;
			curNum[j] = readVal;
//			System.out.println("Adding " + readVal + " to curNum " + " at index " + j);
			j++;
			prevDate = curDate;
		}

 		Stats[0] = (int) sum / Language.monFileSize;
 		Stats[5] = Stats[1] + Stats[2] + Stats[3] + Stats[4];
		return Stats;
		
	}
	
	// Get median from array
	public static double getMedian(double array[], int length){
		if(length <= 0)return -999;
		double[] trunArray = new double[length];
		for(int i = 0; i < length; i++){
			trunArray[i] = array[i];
		}
		java.util.Arrays.sort(trunArray);
//		System.out.println("CurNum is sorted to " + Arrays.toString(trunArray));
		
		if(length <= 1)return -999;

		if(length % 2 == 0){
			return 0.5 * (trunArray[length / 2] + trunArray[length / 2 + 1]);
		}else{
			return trunArray[length / 2];
		}
	}
	
	// This method assumes there are 2-3 numbers before decimal point and returns to 
	// 0 d.p. if two or three digits before decimal 
	// 1 d.p. if one or zero numbers before decimal
	public static String SigFig(double numberD){
		String number = Double.toString(numberD);
		int start = 0;
		start = number.indexOf(".");
		
		if(start == 0)return number;
		
		String wholeNoValue = number.substring(0, start);
		if(wholeNoValue.length() > 1){
			return number.substring(0, start - 1);
		}else{
			return number.substring(start, start + 1);
		}
	}

}


